//------------------------------------------------------------------
#ifndef __API_H
#define __API_H

	#ifdef AMFE_API_EXPORTS
		#define AMFE_API __declspec(dllexport)
	#else
		#define AMFE_API __declspec(dllimport)
	#endif

#endif
//------------------------------------------------------------------
#define AMFE_ERROR_CONNECT_OK											 0
#define AMFE_ERROR_CONNECT_NOT_OPEN								-1
#define AMFE_ERROR_CONNECT_NO_CONFIG							-2
#define AMFE_ERROR_CONNECT_OUT_OF_RANGE						-3
#define AMFE_ERROR_CONNECT_SEND_FAILED						-4
#define AMFE_ERROR_CONNECT_RECEIVED_TOO_LITTLE		-5
#define AMFE_ERROR_CONNECT_I2C_ERROR							-6
#define AMFE_ERROR_CONNECT_BAD_RESPONSE						-7
//------------------------------------------------------------------
typedef struct
{
	__int64		i64MinFreqHz;
	__int64		i64MaxFreqHz;
	__int64		i64LOFreqHz;
	BOOL			bIsInverted;
} TAMFErange;
//------------------------------------------------------------------
typedef struct 
{
	char				szSerialNumber[20];
	BOOL				bIsHS;
	TAMFErange	AMFEranges[5];
} TAMFEconfig;
//------------------------------------------------------------------
extern "C" 
{
	// these functions are used without an open AMFE
	AMFE_API int	GetDeviceCount			(void);																				// create device list and get the count of devices
	AMFE_API BOOL	GetDeviceSerial			(int DeviceIndex, char *serialNumber);

	// these require a valid AMFE handle
	AMFE_API int	Open								(char *serialNumber);														// open the USB device by serial number
	AMFE_API void	Close								(int amfeHandle);															// close the currently open device

	AMFE_API BOOL	GetConfig						(int amfeHandle, TAMFEconfig *lpAmfeConfig);	// get the parameters of currencly open AMFE
	AMFE_API BOOL	IsOpen							(int amfeHandle);															// get if the connection to AMFE is open
	AMFE_API BOOL	IsAlive							(int amfeHandle);															// try the communication if it works

	AMFE_API int	SetRange						(int amfeHandle, DWORD range);								// set the new range from 0 to 4 (or from 0 to 1)
	AMFE_API void	ConvertFrequency		(int amfeHandle, __int64 inputFrequency, __int64 *outputFrequency, DWORD *AMFErange, BOOL *IsInverted);	// convert input frequency to output frequency and amfe range
	AMFE_API BOOL	SetBeep							(int amfeHandle, BOOL DoBeep);								// turn on or off the auto-detection beep
};
//------------------------------------------------------------------
// type defines for function exports from DLLs
	typedef int		(*p_GetDeviceCount)		(void);																						
	typedef BOOL	(*p_GetDeviceSerial)	(int DeviceIndex, char *serialNumber);
	
	typedef int		(*p_Open)							(char *serialNumber);										
	typedef void	(*p_Close)						(int amfeHandle);											

	typedef BOOL	(*p_GetConfig)				(int amfeHandle, TAMFEconfig *lpAmfeConfig);						
	typedef BOOL	(*p_IsOpen)						(int amfeHandle);											
	typedef BOOL	(*p_IsAlive)					(int amfeHandle);																					

	typedef int		(*p_SetRange)					(int amfeHandle, DWORD range);				
	typedef void	(*p_ConvertFrequency)	(int amfeHandle, __int64 inputFrequency, __int64 *outputFrequency, DWORD *AMFErange, BOOL *IsInverted);	
	typedef BOOL	(*p_SetBeep)					(int amfeHandle, BOOL DoBeep);						

//------------------------------------------------------------------