//---------------------------------------------------------------------------
/*         WiNRADiO WR-AMFE8600 example control console application
                          Copyright WiNRADiO 2008
   
 Created using Microsoft Visual C++ 2003 
 (part of the Microsoft Development Environment 2003).

 Recommended text editor settings for the preservation of text formating:
  Tab size: 2
	Indent size: 2

 Note that you need to have the 'AMFE8600_API.DLL' and 'AMFEusb.dll' 
 in the same directory as this application is in. 
*/
//---------------------------------------------------------------------------
#include "stdafx.h"
#include "AMFE8600_API.h"
//-----------------------------------------------------
// function declarations
BOOL LoadAMFEapi(void);																			// load the AMFE8600_API.DLL
void UnloadAMFEapi(void);																		// unload the API from memory
void CloseAMFEconnection(void);															// close the connection to currently open AMFE
BOOL ConnectToAMFE(void);																		// connect to AMFE8600 selected by the user
void ShowMainMenu(void);																		// display the text main menu
void SetBeep(void);																					// turn on or off the AMFE8600 beep function
void SetRange(void);																				// set the AMFE8600 range according to the keyboard input
void SetFrequency(void);																		// set the AMFE8600 range according to the selected frequency
//-----------------------------------------------------
// global variable definitions
HMODULE								AMFEmodule;														// handle of the open AMFE8600_API.DLL
//-----------------------------------------------------
struct 
{
	p_GetDeviceCount							GetDeviceCount;
	p_GetDeviceSerial							GetDeviceSerial;

	p_Open												Open;
	p_Close												Close;

	p_GetConfig										GetConfig;
	p_IsOpen											IsOpen;
	p_IsAlive											IsAlive;

	p_SetRange										SetRange;
	p_ConvertFrequency						ConvertFrequency;
	p_SetBeep											SetBeep;
} amfe_api;

int										amfeHandle;
//-----------------------------------------------------
int _tmain(int argc, _TCHAR* argv[])
{
	BOOL res;																									// BOOLean result
	char key;																									// pressed key
	BOOL terminate = FALSE;																		// a flag which is set when you want to terminate the application

	std::cout << "WR-AMFE8600 API usage example.\nCopyright WiNRADiO 2008.\n\n";
	//-------------------
	// try to load the API
	res = LoadAMFEapi();																			// try to load the AMFE8600_API.DLL and acquire the 2 basic functions

	if(!res)																									// if failed to load the API
	{
		std::cout << "Failed to load WR-AMFE8600 API, terminating.\n";
		return 0;																								// exit
	}
	//-------------------
	// try to connect to one of the connected AMFE8600s
	res = ConnectToAMFE();																		// show the currently connected AMFEs and let the user choose

	if(!res)																									// user did not choose anything or connecting failed
	{
		UnloadAMFEapi();																
		return 0;																								// exit
	}
	//-------------------
	// the main application loop - wait for the command, act by the command and terminate if required
	while(TRUE)																								// a never ending loop (ended by user's input of 'q')
	{
		ShowMainMenu();																					// show the main menu

		key = getch();																					// get the user's choice from the keyboard
		putch(key);																							// display the pressed key back on the screen

		switch(key)																							// act by the command from user
		{
			case 'r': SetRange();				break;										// set the range by range number (0 - 4)
			case 'b': SetBeep();				break;										// turn off or on the AMFE beep
			case 'f': SetFrequency();		break;										// set the range by the entered frequency
			case 'q': terminate = TRUE;	break;										// set the flag that the user wants to quit
		}

		if(terminate)																						// if should terminate the application
			break;																								// get out of the loop
	}
	//-------------------
	// finish the work
	amfe_api.Close(amfeHandle);																// close the connection to AMFE8600
	UnloadAMFEapi();																					// unload the AMFE8600 API
	std::cout << "WR-AMFE8600 API unloaded, terminating.\n";

	return 0;																									// terminate
}
//-----------------------------------------------------
void SetFrequency(void)																			// set the AMFE8600 range according to the selected frequency
{
	float freq;																								// the frequency entered by user which he wants to listen
	int iRes;																									// integer result variable
	__int64 inFreq, outFreq;																	// input and output frequency of AMFE8600
	BOOL isInverted;																					// flag which says if the spectrum will be inverted
	DWORD AMFErange;																					// the AMFE range into which the AMFE8600 will be switched
	double Dfreq;																							// the output frequency converted from Hz to MHz
	char temp[64];																						// buffer used for the formated output of frequency to string

	std::cout << "\n\nEnter the frequency you want to receive [in MHz]: ";

	iRes = scanf("%f", &freq);																// read the input frequency desired by the user from keyboard

	if(iRes != 1)																							// if failed to read the frequency
	{
		std::cout << "\nThe frequency input failed.\n";
		return;																									// exit
	}
	//---------------
	// convert the floating point frequency in MHz to integer frequency in Hz
	inFreq = (__int64) (freq * 1000.0f);											
	inFreq *= 1000;

	// get the frequency which the receiver should be tuned, the new AMFE range and the spectrum inversion flag
	amfe_api.ConvertFrequency(amfeHandle, inFreq, &outFreq, &AMFErange, &isInverted);	

	iRes = amfe_api.SetRange(amfeHandle, AMFErange);					// switch the AMFE8600 to the correct range

	if(iRes != AMFE_ERROR_CONNECT_OK)													// if failed to switch the AMFE
	{
		std::cout << "\nSwitching of AMFE range failed!\n";
		return;																									// exit
	}
	//-------------
	Dfreq = (double) outFreq;																	// convert the frequency from Hz to MHz
	Dfreq = Dfreq / 1000000.0;

	sprintf(temp, "%.3f MHz.\n", (float)Dfreq);								// formated output of the frequency to string

	std::cout << "\nAMFE range changed, tune your receiver to " << temp;

	if(isInverted)																						// if the spectrum is inverted
		std::cout << "Received spectrum IS INVERTED.\n";
	else																											// if the spectrum is not inverted
		std::cout << "Received spectrum is not inverted.\n";
}
//-----------------------------------------------------
void SetBeep(void)																					// turn on or off the AMFE8600 beep function
{
	char key;																									// variable for the pressed key 

	std::cout << "\n\nTurn beeping ON [1] or OFF [0]? ";

	key = getch();																						// get the key and write it back
	putch(key);

	if(key=='1')																							// if the beep should be turned ON
		amfe_api.SetBeep(amfeHandle, TRUE);

	if(key=='0')																							// if the beep should be turned OFF
		amfe_api.SetBeep(amfeHandle, FALSE);

	std::cout << "\n";
}
//-----------------------------------------------------
void SetRange(void)																					// set the AMFE8600 range according to the keyboard input
{
	char key;																									// variable for the pressed key 
	DWORD range;																							// the range to which the AMFE should be switched
	int iRes;																									// variable for integer results

	std::cout << "\n\nPress the range number [0-4] that you want to switch to: ";

	key = getch();																						// get the key and write it back
	putch(key);

	std::cout << "\n";

	if(key < '0' || key > '4')																// if the user's choice is not in the right range
	{
		std::cout << "Your input is invalid.\n";
		return;																									// exit
	}

	range = key - 48;																					// convert the choice from ASCII to number

	iRes = amfe_api.SetRange(amfeHandle, range);							// try to set the range

	switch(iRes)																							// write out the text according to the SetRange result
	{
		case AMFE_ERROR_CONNECT_OK						: std::cout << " OK\n";																				break;
		case AMFE_ERROR_CONNECT_I2C_ERROR			: std::cout << " AMFE device malfunction!\n";									break;
		case AMFE_ERROR_CONNECT_OUT_OF_RANGE	: std::cout << " The selected AMFE range is out of range.\n";	break;
		default																: std::cout << " Communication with AMFE failed\n";						break;
	}
}
//-----------------------------------------------------
void ShowMainMenu(void)																			// just write out the main menu
{
	std::cout << "\nYour options are:\n";
	std::cout << "[r] - switch WR-AMFE8600 range according to entered range\n";
	std::cout << "[f] - switch WR-AMFE8600 range according to entered frequency\n";
	std::cout << "[b] - turn on or off the beeping\n";
	std::cout << "[q] - quit\n\n";

	std::cout << "Enter your choice now: ";
}
//-----------------------------------------------------
BOOL ConnectToAMFE(void)																					// connect to AMFE8600 selected by the user
{
	DWORD FoundDevs;																								// number of found AMFE8600 on USB, 
	DWORD i;																												// variable used for loop counting
	int index;																											// index of the selected AMFE8600 device
	char key;																												// the pressed key
	char usbSerial[20];

	while(TRUE)																											// the infinite loop
	{
		FoundDevs = amfe_api.GetDeviceCount();												// look on the USB bus for the devices and return count

		std::cout << "\nWR-AMFE8600 devices found: " << FoundDevs << "\n";

		if(FoundDevs)																									// if some devices were found
		{
			// get the serial numbers
			for(i=0; i<FoundDevs; i++)																	// write out the list of present AMFE8600 devices
			{
				amfe_api.GetDeviceSerial(i, usbSerial);
				std::cout << "[" << i << "] - " << usbSerial << "\n";
			}
		}
		else																													// if no AMFE8600 device was found on the USB
		{
			std::cout << "Press any key to search again for some WR-AMFE8600.\n";
			key = getch();																							// wait for any key
			continue;																										// and try again
		}
		//---------------------
		// now we got the list of devices displayed, let the user choose one
		std::cout << "Press WR-AMFE8600 number (0-" << (FoundDevs-1) << ") which you want to open or any other key to quit: ";
		
		key = getch();																								// get the user's choice

		if(key < '0' || key > '9')																		// if user pressed anything other than number
		{
			std::cout << "Other key was pressed, terminating.\n";
			return FALSE;																								// terminate
		}
		
		index = key - 48;																							// convert ASCII key to number

		if(index >= 0 && index < (int)FoundDevs)											// if the number is in the range of list of AMFE8600 devices
			break;																											// move forward then
	}
	//--------------
	// open the AMFE8600 by it's serial number
	amfe_api.GetDeviceSerial(index, usbSerial);
	std::cout << "Opening WR-AMFE8600 with serial number " << usbSerial << ": ";

	amfeHandle = amfe_api.Open(usbSerial);													// open MFE8600 by serial number					

	if(amfeHandle == -1)																						// if failed to open
	{
		std::cout << "failed, terminating.\n";
		return FALSE;																									// exit
	}

	std::cout << "OK\n";
	return TRUE;																										// we succeeded
}
//-----------------------------------------------------
BOOL LoadAMFEapi(void)																						// load the AMFE8600_API.DLL
{
	char *bfr = new char[1024];																			// allocate buffer for the path to AMFE8600_API.DLL
	DWORD FileNameSize;																							// variable for the size of the path
	DWORD i;																												// variable used in the 'for' loop

	//----------
	// 1st - we need to determine the path to this currently running application
	FileNameSize = GetModuleFileName(NULL, bfr, 1024);							// get the path and length of this executable's file path 

	if(FileNameSize == 1024)																				// if buffer wasn't big enough and the string was truncated
	{
		FileNameSize = 10240;																					// make the buffer really big

		delete []bfr;																									// delete the previous buffer
		char *bfr = new char[FileNameSize];														// and allocate a new one

		FileNameSize = GetModuleFileName(NULL, bfr, FileNameSize);		// get the path and length of this executable's file path 
	}

	if(!FileNameSize)																								// if failed to get the current module's path
	{
		delete []bfr;																									// delete the buffer
		return FALSE;																									// exit with failure
	}
	//----------
	// 2nd - we need to find the '\' between the path and the filename
	for(i=FileNameSize-1; i>0; i--)																	// find the start of filename
		if(bfr[i] == '\\')
			break;

	// replace the filename of the application with the filename of the API
	strcpy(&bfr[i+1], "AMFE8600_API.DLL");													
	
	//-------------
	// 3rd - we need to load the library
	AMFEmodule = LoadLibrary(bfr);																	// try to load the library
	delete []bfr;																										// deallocate the buffer

	if(!AMFEmodule)																									// if the LoadLibrary failed, the AMFE_API.DLL is probably missing
		return FALSE;																									// exit with failure

	//-------------
	// 4th - we need to get the addresses of the two exported functions from the library
	amfe_api.GetDeviceCount				= (p_GetDeviceCount)		GetProcAddress(AMFEmodule, "GetDeviceCount");
	amfe_api.GetDeviceSerial			= (p_GetDeviceSerial)		GetProcAddress(AMFEmodule, "GetDeviceSerial");

	amfe_api.Open									= (p_Open)							GetProcAddress(AMFEmodule, "Open");
	amfe_api.Close								= (p_Close)							GetProcAddress(AMFEmodule, "Close");

	amfe_api.GetConfig						= (p_GetConfig)					GetProcAddress(AMFEmodule, "GetConfig");
	amfe_api.IsOpen								= (p_IsOpen)						GetProcAddress(AMFEmodule, "IsOpen");
	amfe_api.IsAlive							= (p_IsAlive)						GetProcAddress(AMFEmodule, "IsAlive");

	amfe_api.SetRange							= (p_SetRange)					GetProcAddress(AMFEmodule, "SetRange");
	amfe_api.ConvertFrequency			= (p_ConvertFrequency)	GetProcAddress(AMFEmodule, "ConvertFrequency");
	amfe_api.SetBeep							= (p_SetBeep)						GetProcAddress(AMFEmodule, "SetBeep");


	if(	!amfe_api.GetDeviceCount	|| !amfe_api.GetDeviceSerial	|| !amfe_api.Open			|| !amfe_api.Close			|| !amfe_api.IsOpen ||
			!amfe_api.SetRange				|| !amfe_api.ConvertFrequency	|| !amfe_api.SetBeep	|| !amfe_api.GetConfig	|| !amfe_api.IsAlive )
	{
		FreeLibrary(AMFEmodule);																			// unload the DLL from memory
		return FALSE;																									// exit with failure
	}
	//-----------
	return TRUE;																										// exit with success
}
//----------------------------------------------------
void UnloadAMFEapi(void)																					// unload the API from memory
{
	if(amfe_api.Close)																							// if the AMFE instance exists
		CloseAMFEconnection();																				// try to close the connection to AMFE
	//----------------------
	if(AMFEmodule)																									// if we go the handle to AMFE_API.dll
		FreeLibrary(AMFEmodule);																			// unload the .dll

	memset(&amfe_api, 0, sizeof(amfe_api));
	amfeHandle = -1;
}
//----------------------------------------------------
void CloseAMFEconnection(void)																		// closes the connection to AMFE
{
	if(amfe_api.Close)																							// if the AMFE object exists, close the connection to AMFE
		amfe_api.Close(amfeHandle);
}
//----------------------------------------------------

